/*
 * Web site error daemon - network socket handler
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#include "library.h"
#include "vector.h"
#include "utils.h"
#include "log.h"
#include "runtime.h"

#define BUFLEN BUFSIZ	/* 8192 on Linux glibc-2.0.7pre6 */

void
recv_recv(vector_t *runtime, int fd)
{
	int					 nbytes;
	errmsg_t			 msg;
	struct sockaddr_in	 from;
	int					 fromlen = sizeof(from);

	memset(&msg, 0, sizeof(msg));
	nbytes = recvfrom(fd, ((void *)(&msg)), sizeof(msg), 0,
			((struct sockaddr *)(&from)), &fromlen);

	if (nbytes < 0) {
		perror("recvmesg");
		fprintf(stderr, "Got no bytes\n");
		return;
	}
	if (nbytes != sizeof(msg)) {
		fprintf(stderr, "Got %d bytes, expected %d\n",
						nbytes, sizeof(msg));
	}

	/* This is safe */
	msg.version = ntohl(msg.version);
	msg.pid = ntohl(msg.pid);
	msg.site[(LOG_SITELEN - 1)] = 0;
	msg.port = ntohl(msg.port);
	msg.referer[(LOG_REFERERLEN - 1)] = 0;
	msg.url[(LOG_URLLEN - 1)] = 0;
	msg.program[(LOG_PROGRAMLEN - 1)] = 0;
	msg.facility[(LOG_FACILITYLEN - 1)] = 0;
	msg.priority = ntohl(msg.priority);
	msg.message[(LOG_MESSAGELEN - 1)] = 0;

	printf("Got a message\n");
#if 0
	printf("Version: %ld\n", msg.version);
	printf("PID: %ld\n", msg.pid);
	printf("Site: %s\n", msg.site);
	printf("Port: %ld\n", msg.port);
	printf("Referer: %s\n", msg.referer);
	printf("URL: %s\n", msg.url);
	printf("Program: %s\n", msg.program);
	printf("Facility: %s\n", msg.facility);
	printf("Priority: %ld\n", msg.priority);
	printf("Message: (%d) %s\n", strlen(msg.message), msg.message);
#endif
	runtime_execute(runtime, &msg);
}

int
recv_bind(int port)
{
	struct sockaddr_in	 saddr;
	struct protoent		*proto;
	int					 sock;

	proto = getprotobyname("udp");
	sock = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
	if (sock < 0) {
		perror("socket");
		exit(-1);
	}

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);
	saddr.sin_addr.s_addr = htonl(INADDR_ANY);

	if (bind(sock, ((struct sockaddr *)(&saddr)), sizeof(saddr)) < 0) {
		perror("bind");
		exit(-1);
	}
	printf("Socket bound to port %d\n", port);

#if 0
	if (listen(sock, 5) < 0) {
		perror("listen");
		exit(-1);
	}
	printf("Listening for connections\n");
#endif
	
	return sock;
}
