/*
 * Random image selector
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <time.h>
#include <dirent.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "smalloc.h"
#include "library.h"
#include "vector.h"
#include "cgi.h"
#include "dirvec.h"
#include "hash.h"
#include "rope.h"
#include "log.h"

#define DIR_RANDOMIMAGE DIR_DATA "/"
#define DIR_LOCATION	DIR_RESOURCE "/randomimage/images/"

#undef HUGE_SECURITY_HOLE
/* This define is a OW backwards-compatibility hack. Don't define
 * it. Just use the new method of specifying arguments. Better,
 * use the billboard CGI with Apache-SSI. */

static char *
get_filename(char *path)
{
	time_t		 timebuf;
	long		 ran;	/* Modular is OK because of huge size */
	vector_t	*v;

	v = dirvec_new(path);
	if (!v) {
		log("cgisys", PRI_CRIT, "No vector returned from "
						"dirvec_new(%s)", path);
		return NULL;
	}
	if (!vector_length(v)) {
		log("cgiconf", PRI_WARNING, "No images in '%s'", path);
		return NULL;
	}

	time(&timebuf);
	srandom(timebuf);
	ran = (random() >> 4) % vector_length(v);

	return vector_data(v, ran);

}

#ifdef HUGE_SECURITY_HOLE
void
old_method(hash_t *cgihash, char *host)
{

	char	*path;
	char	*dir;
	char	*image;

	dir = hash_get(cgihash, "ImageDirectoryURL");

	if (!dir) {
		log("cgiconf", PRI_CRIT, "No 'ImageDirectoryURL' field in "
						"CGI hash table");
		return;
	}

	/* XXX IS THIS ABSOLUTE? Tony says it is. */
	path = hash_get(cgihash, "ImageDirectoryPath");
	if (!path) {
		log("cgiconf", PRI_WARNING, "No 'Path' field in hash table");
		return;
	}

	image = get_filename(path);
	if (image)
		printf("Location: http://%s/%s/%s\n\n", host, dir, image);
}
#endif

void
new_method(char *host, char *dir)
{
	rope_t	*r;
	char	*image;

	dir[strcspn(dir, "/")] = 0;		/* Secure me */
	host[strcspn(host, "/")] = 0;	/* Secure me */

	r = rope_new(DIR_RANDOMIMAGE);
	rope_add(r, host);
	rope_add(r, "/randomimage/images/");
	rope_add(r, dir);
	image = get_filename(rope_chars(r));
	printf("Location: http://%s" DIR_LOCATION "%s/%s\n\n",
					host, dir, image);
}

void
invoke(void)
{
	hash_t		*cgihash;
	char		*host;
	char		*dir;

	host = getenv(ENV_HOSTNAME);
	if (!host) {
		log("cgisys", PRI_CRIT, "Randomimage got no " ENV_HOSTNAME);
		exit(0);
	}

	cgihash = cgi_hash_new();
	dir = hash_get(cgihash, "case");

	if (dir) {
		new_method(host, dir);
	}
#ifdef HUGE_SECURITY_HOLE
	else {
		old_method(cgihash, host);
	}
#else
	else {
		log("cgiconf", PRI_CRIT, "No 'case' parameter in QUERY_STRING");
	}
#endif

}
